 # openapi-embyapi

A Unofficial Python client library for accessing Emby Server REST API which is automatically generated by the [openapi-python-client](https://github.com/openapi-generators/openapi-python-client) :

- API version: beta (current version is 4.7.12.0)
- Build package version: latest (current version is 0.14.1)

## Why This?

This client focuses on creating the best developer experience for Python developers to access Emby Server REST API by:
1. Using asyncio and httpx.
2. Using all the latest and greatest Python features like type annotations and dataclasses. 

## Requirements

python >= 3.8  

### Installation

I recommend you install with [Poetry](https://python-poetry.org/docs/) so you can easily manage your dependency and virtual environment:

    cd openapi-python-embyapi
    poetry init
    poetry install 

You can also install with normal pip: `pip install openapi-python-embyapi`

### Usage

First, create a client using `AuthenticatedClient`:

```python
from openapi_python_embyapi import AuthenticatedClient

client = AuthenticatedClient(base_url="https://api.example.com", token="SuperSecretToken")
```

Now call your endpoint and use your models with an async version:

```python
from openapi_python_embyapi.models import MyDataModel
from openapi_python_embyapi.api.my_tag import get_my_data_model
from openapi_python_embyapi.types import Response

my_data: MyDataModel = await get_my_data_model.asyncio(client=client)
response: Response[MyDataModel] = await get_my_data_model.asyncio_detailed(client=client)
```

There are more settings on the generated `Client` class which let you control more runtime behavior, check out the docstring on that class for more info.

Things to know:
1. Every path/method combo becomes a Python module with four functions:
    1. `sync`: Blocking request that returns parsed data (if successful) or `None`
    2. `sync_detailed`: Blocking request that always returns a `Request`, optionally with `parsed` set if the request was successful.
    3. `asyncio`: Like `sync` but async instead of blocking
    4. `asyncio_detailed`: Like `sync_detailed` but async instead of blocking

2. All path/query params, and bodies become method arguments.
3. If your endpoint had any tags on it, the first tag will be used as a module name for the function (my_tag above)
4. Any endpoint which did not have a tag will be in `openapi_python_embyapi.api.default`