from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.dlna_dlna_profile_type import DlnaDlnaProfileType
from ..models.dlna_encoding_context import DlnaEncodingContext
from ..models.dlna_transcode_seek_info import DlnaTranscodeSeekInfo
from ..types import UNSET, Unset

T = TypeVar("T", bound="DlnaTranscodingProfile")


@attr.s(auto_attribs=True)
class DlnaTranscodingProfile:
    """
    Attributes:
        container (Union[Unset, str]):
        type (Union[Unset, DlnaDlnaProfileType]):
        video_codec (Union[Unset, str]):
        audio_codec (Union[Unset, str]):
        protocol (Union[Unset, str]):
        estimate_content_length (Union[Unset, bool]):
        enable_mpegts_m2_ts_mode (Union[Unset, bool]):
        transcode_seek_info (Union[Unset, DlnaTranscodeSeekInfo]):
        copy_timestamps (Union[Unset, bool]):
        context (Union[Unset, DlnaEncodingContext]):
        max_audio_channels (Union[Unset, str]):
        min_segments (Union[Unset, int]):
        segment_length (Union[Unset, int]):
        break_on_non_key_frames (Union[Unset, bool]):
        allow_interlaced_video_stream_copy (Union[Unset, bool]):
        manifest_subtitles (Union[Unset, str]):
        max_manifest_subtitles (Union[Unset, int]):
    """

    container: Union[Unset, str] = UNSET
    type: Union[Unset, DlnaDlnaProfileType] = UNSET
    video_codec: Union[Unset, str] = UNSET
    audio_codec: Union[Unset, str] = UNSET
    protocol: Union[Unset, str] = UNSET
    estimate_content_length: Union[Unset, bool] = UNSET
    enable_mpegts_m2_ts_mode: Union[Unset, bool] = UNSET
    transcode_seek_info: Union[Unset, DlnaTranscodeSeekInfo] = UNSET
    copy_timestamps: Union[Unset, bool] = UNSET
    context: Union[Unset, DlnaEncodingContext] = UNSET
    max_audio_channels: Union[Unset, str] = UNSET
    min_segments: Union[Unset, int] = UNSET
    segment_length: Union[Unset, int] = UNSET
    break_on_non_key_frames: Union[Unset, bool] = UNSET
    allow_interlaced_video_stream_copy: Union[Unset, bool] = UNSET
    manifest_subtitles: Union[Unset, str] = UNSET
    max_manifest_subtitles: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        container = self.container
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        video_codec = self.video_codec
        audio_codec = self.audio_codec
        protocol = self.protocol
        estimate_content_length = self.estimate_content_length
        enable_mpegts_m2_ts_mode = self.enable_mpegts_m2_ts_mode
        transcode_seek_info: Union[Unset, str] = UNSET
        if not isinstance(self.transcode_seek_info, Unset):
            transcode_seek_info = self.transcode_seek_info.value

        copy_timestamps = self.copy_timestamps
        context: Union[Unset, str] = UNSET
        if not isinstance(self.context, Unset):
            context = self.context.value

        max_audio_channels = self.max_audio_channels
        min_segments = self.min_segments
        segment_length = self.segment_length
        break_on_non_key_frames = self.break_on_non_key_frames
        allow_interlaced_video_stream_copy = self.allow_interlaced_video_stream_copy
        manifest_subtitles = self.manifest_subtitles
        max_manifest_subtitles = self.max_manifest_subtitles

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if container is not UNSET:
            field_dict["Container"] = container
        if type is not UNSET:
            field_dict["Type"] = type
        if video_codec is not UNSET:
            field_dict["VideoCodec"] = video_codec
        if audio_codec is not UNSET:
            field_dict["AudioCodec"] = audio_codec
        if protocol is not UNSET:
            field_dict["Protocol"] = protocol
        if estimate_content_length is not UNSET:
            field_dict["EstimateContentLength"] = estimate_content_length
        if enable_mpegts_m2_ts_mode is not UNSET:
            field_dict["EnableMpegtsM2TsMode"] = enable_mpegts_m2_ts_mode
        if transcode_seek_info is not UNSET:
            field_dict["TranscodeSeekInfo"] = transcode_seek_info
        if copy_timestamps is not UNSET:
            field_dict["CopyTimestamps"] = copy_timestamps
        if context is not UNSET:
            field_dict["Context"] = context
        if max_audio_channels is not UNSET:
            field_dict["MaxAudioChannels"] = max_audio_channels
        if min_segments is not UNSET:
            field_dict["MinSegments"] = min_segments
        if segment_length is not UNSET:
            field_dict["SegmentLength"] = segment_length
        if break_on_non_key_frames is not UNSET:
            field_dict["BreakOnNonKeyFrames"] = break_on_non_key_frames
        if allow_interlaced_video_stream_copy is not UNSET:
            field_dict["AllowInterlacedVideoStreamCopy"] = allow_interlaced_video_stream_copy
        if manifest_subtitles is not UNSET:
            field_dict["ManifestSubtitles"] = manifest_subtitles
        if max_manifest_subtitles is not UNSET:
            field_dict["MaxManifestSubtitles"] = max_manifest_subtitles

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        container = d.pop("Container", UNSET)

        _type = d.pop("Type", UNSET)
        type: Union[Unset, DlnaDlnaProfileType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = DlnaDlnaProfileType(_type)

        video_codec = d.pop("VideoCodec", UNSET)

        audio_codec = d.pop("AudioCodec", UNSET)

        protocol = d.pop("Protocol", UNSET)

        estimate_content_length = d.pop("EstimateContentLength", UNSET)

        enable_mpegts_m2_ts_mode = d.pop("EnableMpegtsM2TsMode", UNSET)

        _transcode_seek_info = d.pop("TranscodeSeekInfo", UNSET)
        transcode_seek_info: Union[Unset, DlnaTranscodeSeekInfo]
        if isinstance(_transcode_seek_info, Unset):
            transcode_seek_info = UNSET
        else:
            transcode_seek_info = DlnaTranscodeSeekInfo(_transcode_seek_info)

        copy_timestamps = d.pop("CopyTimestamps", UNSET)

        _context = d.pop("Context", UNSET)
        context: Union[Unset, DlnaEncodingContext]
        if isinstance(_context, Unset):
            context = UNSET
        else:
            context = DlnaEncodingContext(_context)

        max_audio_channels = d.pop("MaxAudioChannels", UNSET)

        min_segments = d.pop("MinSegments", UNSET)

        segment_length = d.pop("SegmentLength", UNSET)

        break_on_non_key_frames = d.pop("BreakOnNonKeyFrames", UNSET)

        allow_interlaced_video_stream_copy = d.pop("AllowInterlacedVideoStreamCopy", UNSET)

        manifest_subtitles = d.pop("ManifestSubtitles", UNSET)

        max_manifest_subtitles = d.pop("MaxManifestSubtitles", UNSET)

        dlna_transcoding_profile = cls(
            container=container,
            type=type,
            video_codec=video_codec,
            audio_codec=audio_codec,
            protocol=protocol,
            estimate_content_length=estimate_content_length,
            enable_mpegts_m2_ts_mode=enable_mpegts_m2_ts_mode,
            transcode_seek_info=transcode_seek_info,
            copy_timestamps=copy_timestamps,
            context=context,
            max_audio_channels=max_audio_channels,
            min_segments=min_segments,
            segment_length=segment_length,
            break_on_non_key_frames=break_on_non_key_frames,
            allow_interlaced_video_stream_copy=allow_interlaced_video_stream_copy,
            manifest_subtitles=manifest_subtitles,
            max_manifest_subtitles=max_manifest_subtitles,
        )

        dlna_transcoding_profile.additional_properties = d
        return dlna_transcoding_profile

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
